# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.

# SPDX-FileCopyrightText: 2023 LLVM Project Contributors
#
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

function(build_vfs_overlay winsdk_src)
    set(vfs_dest "${winsdk_src}/vfs_overlay.yaml")
    if(NOT EXISTS "${vfs_dest}")
        unset(include_dirs)
        file(
            GLOB_RECURSE entries
            LIST_DIRECTORIES true
            "${winsdk_src}/*")
        foreach(entry ${entries})
            if(IS_DIRECTORY "${entry}")
                list(APPEND include_dirs "${entry}")
            endif()
        endforeach()

        file(WRITE "${vfs_dest}" "version: 0\ncase-sensitive: false\nroots:\n")

        foreach(include_dir ${include_dirs})
            file(
                GLOB headers
                RELATIVE "${include_dir}"
                "${include_dir}/*.h")
            if(NOT headers)
                continue()
            endif()

            file(APPEND "${vfs_dest}" "  - name: \"${include_dir}\"\n    type: directory\n    contents:\n")
            foreach(header ${headers})
                file(
                    APPEND "${vfs_dest}"
                    "      - name: \"${header}\"\n        type: file\n        external-contents: \"${include_dir}/${header}\"\n"
                )
            endforeach()
        endforeach()
    endif()

	set(CMAKE_C_FLAGS_INIT
		"${CMAKE_C_FLAGS_INIT} -Xclang -ivfsoverlay -Xclang \"${vfs_dest}\""
		PARENT_SCOPE)
	set(CMAKE_CXX_FLAGS_INIT
		"${CMAKE_CXX_FLAGS_INIT} -Xclang -ivfsoverlay -Xclang \"${vfs_dest}\""
		PARENT_SCOPE)
    set(CMAKE_RC_FLAGS_INIT
        "${CMAKE_RC_FLAGS_INIT} -Xclang -ivfsoverlay -Xclang \"${vfs_dest}\""
        PARENT_SCOPE)
endfunction()

function(process_library_path winsdk_src)
    set(winsdk_dest "${winsdk_src}-lc")
    if(NOT EXISTS "${winsdk_dest}")
        execute_process(COMMAND "${CMAKE_COMMAND}" -E make_directory "${winsdk_dest}")
        file(
            GLOB lib_list
            RELATIVE "${winsdk_src}"
            "${winsdk_src}/*")
        foreach(lib_src ${lib_list})
            string(TOLOWER "${lib_src}" lib_dest)
            if(NOT "${lib_src}" STREQUAL "${lib_dest}")
                execute_process(COMMAND "${CMAKE_COMMAND}" -E create_symlink "${winsdk_src}/${lib_src}"
                                        "${winsdk_dest}/${lib_dest}")
            endif()
        endforeach()
    endif()

    link_directories("${winsdk_src}" "${winsdk_dest}")
endfunction()
