# SPDX-FileCopyrightText: 2023 Daniel Wolf <nephatrine@gmail.com>
#
# SPDX-License-Identifier: ISC

include(/opt/llvm-msvc/share/nxbuild/MSVCCompileFunctions.cmake)

set(CMAKE_SYSTEM_NAME "Windows")
set(CMAKE_SYSTEM_PROCESSOR "x86_64")
set(CMAKE_SYSTEM_VERSION "$ENV{WindowsSDKVersion}")
set(CMAKE_VS_PLATFORM_NAME "x64")
set(CMAKE_SIZEOF_VOID_P 8)

set(triplet "${CMAKE_SYSTEM_PROCESSOR}-pc-windows-msvc")

set(CMAKE_SYSROOT "$ENV{WINDOWS_SYSROOT}")
list(APPEND CMAKE_PREFIX_PATH "$ENV{WINDOWS_TOOLCHAIN}" "/usr/lib/llvm-$ENV{LLVM_MAJOR}")

find_program(CMAKE_C_COMPILER NAMES "clang-cl-$ENV{LLVM_MAJOR}" "clang-cl")
find_program(CMAKE_CXX_COMPILER NAMES "clang-cl-$ENV{LLVM_MAJOR}" "clang-cl")
find_program(CMAKE_RC_COMPILER NAMES "llvm-rc-$ENV{LLVM_MAJOR}" "llvm-rc")

find_program(CMAKE_MT NAMES "llvm-mt-$ENV{LLVM_MAJOR}" "llvm-mt")
find_program(CMAKE_AR NAMES "llvm-lib-$ENV{LLVM_MAJOR}" "llvm-lib")
find_program(CMAKE_LINKER NAMES "lld-link-$ENV{LLVM_MAJOR}" "lld-link")

set(CMAKE_LIBRARY_ARCHITECTURE "${CMAKE_VS_PLATFORM_NAME}")
set(CMAKE_C_LIBRARY_ARCHITECTURE "${CMAKE_LIBRARY_ARCHITECTURE}")
set(CMAKE_CXX_LIBRARY_ARCHITECTURE "${CMAKE_LIBRARY_ARCHITECTURE}")
set(CMAKE_ASM_LIBRARY_ARCHITECTURE "${CMAKE_LIBRARY_ARCHITECTURE}")

set(MSVC_C_ARCHITECTURE_ID "${CMAKE_VS_PLATFORM_NAME}")
set(MSVC_CXX_ARCHITECTURE_ID "${CMAKE_VS_PLATFORM_NAME}")
set(MSVC_ASM_ARCHITECTURE_ID "${CMAKE_VS_PLATFORM_NAME}")

string(REPLACE "." ";" lsVSTools "$ENV{VCToolsVersion}")
list(GET lsVSTools 0 iVCMajor)
list(GET lsVSTools 1 iVCMinor)
math(EXPR iVCMajor "${iVCMajor} + 5")

set(CMAKE_C_COMPILER_TARGET "${triplet}")
set(CMAKE_CXX_COMPILER_TARGET "${triplet}")
set(CMAKE_ASM_COMPILER_TARGET "${triplet}")

set(CMAKE_C_FLAGS_INIT "--target=${CMAKE_C_COMPILER_TARGET} -fms-compatibility-version=${iVCMajor}.${iVCMinor} -vctoolsversion $ENV{VCToolsVersion} -winsdkversion $ENV{WindowsSDKVersion}")
set(CMAKE_CXX_FLAGS_INIT "--target=${CMAKE_CXX_COMPILER_TARGET} -fms-compatibility-version=${iVCMajor}.${iVCMinor} -vctoolsversion $ENV{VCToolsVersion} -winsdkversion $ENV{WindowsSDKVersion}")
set(CMAKE_ASM_FLAGS_INIT "--target=${CMAKE_ASM_COMPILER_TARGET} -fms-compatibility-version=${iVCMajor}.${iVCMinor} -vctoolsversion $ENV{VCToolsVersion} -winsdkversion $ENV{WindowsSDKVersion}")

set(CMAKE_INCLUDE_SYSTEM_FLAG_C "-imsvc")
set(CMAKE_INCLUDE_SYSTEM_FLAG_CXX "-imsvc")

set(CMAKE_EXE_LINKER_FLAGS_INIT "/manifest:no /machine:X64")
set(CMAKE_MODULE_LINKER_FLAGS_INIT "${CMAKE_MODULE_LINKER_FLAGS_INIT}")
set(CMAKE_SHARED_LINKER_FLAGS_INIT "${CMAKE_MODULE_LINKER_FLAGS_INIT}")
set(CMAKE_STATIC_LINKER_FLAGS_INIT "/machine:X64")

set(CMAKE_USER_MAKE_RULES_OVERRIDE "/opt/llvm-msvc/share/nxbuild/MSVCCompileRules.cmake")

include_directories(
    SYSTEM "$ENV{VCToolsInstallDir}/include" "$ENV{WindowsSdkDir}/Include/$ENV{WindowsSDKVersion}/ucrt"
    "$ENV{WindowsSdkDir}/Include/$ENV{WindowsSDKVersion}/shared" "$ENV{WindowsSdkDir}/Include/$ENV{WindowsSDKVersion}/um"
    "$ENV{WindowsSdkDir}/Include/$ENV{WindowsSDKVersion}/winrt" "$ENV{WindowsSdkDir}/Include/$ENV{WindowsSDKVersion}/cppwinrt")

build_vfs_overlay("$ENV{WindowsSdkDir}/Include")
process_library_path("$ENV{VCToolsInstallDir}/lib/${CMAKE_VS_PLATFORM_NAME}")
process_library_path("$ENV{WindowsSdkDir}/Lib/$ENV{WindowsSDKVersion}/ucrt/${CMAKE_VS_PLATFORM_NAME}")
process_library_path("$ENV{WindowsSdkDir}/Lib/$ENV{WindowsSDKVersion}/um/${CMAKE_VS_PLATFORM_NAME}")

find_program(CMAKE_CROSSCOMPILING_EMULATOR NAMES "$ENV{WINE}" "wine64" "wine")
set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM BOTH)
set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_PACKAGE ONLY)
